"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var fs = require("fs");
var program = require("commander");
var ora = require("ora");
var chalk_1 = require("chalk");
var PrettyError = require("pretty-error");
var pkg = require("pjson");
var semver = require("semver");
var honoka_1 = require("honoka");
var generator = require("./generator");
var Config_1 = require("./Config");
var registerHonoka_1 = require("./registerHonoka");
module.exports = (function () {
    function MuseGeneratorCli() {
        registerHonoka_1.default();
        this.pe = new PrettyError();
        this.spinners = {
            update: ora('Checking for Updates'),
            fetching: ora('Fetching from Netease')
        };
    }
    MuseGeneratorCli.prototype.start = function () {
        var _this = this;
        var input = [];
        program
            .version(pkg.version)
            .option('-c, --check', 'check for updates')
            .option('-t, --temporary', 'temporary mode')
            .option('-s, --stdout', 'stdout mode')
            .parse(process.argv);
        if (program.check) {
            this.checkUpdate().catch(function () {
                _this.spinners.update.stop();
            });
            return;
        }
        if (program.args.length === 0) {
            program.outputHelp();
        }
        else {
            if (program.args.length === 1) {
                input = program.args[0].toString().split(',');
            }
            else if (program.args.length > 1) {
                input = program.args;
            }
            if (program.temporary) {
                input.push({ temporary: true });
            }
            this.spinners.fetching.start();
            generator.apply(void 0, input).then(function (playlist) {
                _this.spinners.fetching.stop();
                if (!program.stdout) {
                    fs.writeFileSync('playlist.json', playlist);
                    console.log(chalk_1.default.bold.green('playlist.json generated successfully'));
                }
                else {
                    console.log(playlist);
                }
            })
                .catch(function (err) {
                _this.spinners.fetching.stop();
                console.log(_this.pe.render(err));
            });
        }
    };
    MuseGeneratorCli.prototype.checkUpdate = function () {
        return __awaiter(this, void 0, void 0, function () {
            var latestPkg, latestVersion, currentVersion;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.spinners.update.start();
                        return [4, honoka_1.default.get(Config_1.default.PackageURL)];
                    case 1:
                        latestPkg = _a.sent();
                        this.spinners.update.stop();
                        latestPkg = JSON.parse(latestPkg);
                        latestVersion = latestPkg.version;
                        currentVersion = pkg.version;
                        if (!semver.valid(latestVersion)) {
                            return [2];
                        }
                        if (semver.gt(latestVersion, currentVersion)) {
                            console.log(chalk_1.default.bold.red("Your current version of " + pkg.name + " is out of date. The latest version is " + latestVersion + " while you're on " + currentVersion + "."));
                        }
                        else {
                            console.log(chalk_1.default.bold.green("You're on latest version " + currentVersion));
                        }
                        return [2];
                }
            });
        });
    };
    return MuseGeneratorCli;
}());
